<?php

namespace Schaffen\SerasaRemessa\Utils;

class Record {

    /**
     * @var array Lista de colunas do registro
     */
    private $columns = [];

    /**
     * @var array Template para o carregamento automaático de colunas
     */
    protected $templateColumns = [];

    /**
     * @param array | null  $values lista de valores no formato, chave => valor
     */
    public function __construct($values = null) {
        // cria as colunas a partir do $templateColumns
        $this->createColumns();
        // seta os valores das colunas
        if($values !== null) {
            $this->setColumnsValues($values);
        }
    }


    /**
     * Define o valore de uma coluna
     * @param string $name Nome do campo
     * @param RecordColumn $recordColumn Objeto da coluna
     */
    protected function setColumn($name, RecordColumn $recordColumn) {
        $this->columns[$name] = $recordColumn;
    }

    /**
     * Define as colunas dinamicamente a partir de $templateColumns
     */
    protected function createColumns() {
        foreach($this->templateColumns as $column) {
            $this->setColumn($column['name'], new RecordColumn(null, $column['type'], $column['position'], $column['size']));
        }
    }

    /**
     * Define os valores das colunas 
     * @param array $values Lista de valoes das colunas no formato chave => valor
     */
    protected function setColumnsValues($values) {
        foreach($values as $key => $value) {
            if(isset($this->columns[$key])) {
                $this->getColumn($key)->setValue($value);
            }
        }
    }

    /**
     * Carrega os valores das colunas dinamicamente
     * @param string $recordString String do registro
     */
    public function load($recordString) {
        foreach($this->templateColumns as $column) {
            $value = substr($recordString, $column['position'] - 1, $column['size']);
            $this->getColumn($column['name'])->setValue($value);
        }
    }

    /**
     * @return string Registro formatado
     */
    public function toString() {
        $recordString = '';
        foreach($this->columns as $column){
            $recordString = substr_replace($recordString, $column->getFormatedValue(), $column->getPosition() - 1, 0);
        }

        return $recordString;
    }

    /**
     * Retorna uma coluna
     * @param string $name
     * @return RecordColumn Coluna requisitada
     */
    public function getColumn($name) {
        if(!isset($this->columns[$name])) {
            throw new \Exception("Column '{$name}' does not exists"); 
        }

        return $this->columns[$name];
    }

    /**
     * Retorna todos os valores do registro em uma lista no formato chave => valor
     * @return array Valores 
     */
    public function getValues() {
        return array_map(function($item) {
            return $item->getValue();
        }, $this->columns);
    }
}